@@ -41,9 +41,9 @@ void reset_console(void)
 {
 	if (!config.dumbterm) {
 		write(app.pstdout->cfhno, app.pstdout->escreset,
-		      strlen(app.pstdout->escreset));
+		      app.pstdout->escresetlen);
 		write(app.pstderr->cfhno, app.pstderr->escreset,
-		      strlen(app.pstderr->escreset));
+		      app.pstderr->escresetlen);
 	}
 
 	if (app.ptytermios_bak != NULL)
@@ -107,12 +107,17 @@ int print_stream(pipe_t *pipe)
 			offset = 0;
 			buf = pipe->esccolor;
 			esclength = pipe->buf - pipe->esccolor;
-			if (!regmatch) {
-				*pipe->bgesccolor = pipe->bgcol + '0';
-				pipe_old = pipe;
+			if (pipe->bgesccolor) {
+				if (!regmatch) {
+					*pipe->bgesccolor = pipe->bgcol + '0';
+					pipe_old = pipe;
+				} else {
+					*pipe->bgesccolor = pipe->regbgcol
+						+ '0';
+					pipe_old = NULL;
+				}
 			} else {
-				*pipe->bgesccolor = pipe->regbgcol + '0';
-				pipe_old = NULL;
+				pipe_old = pipe;
 			}
 		} else {
 			buf = pipe->buf;
@@ -123,7 +128,7 @@ int print_stream(pipe_t *pipe)
 
 	/* Clip output if configured */
 	if (pipe->eclip) {
-		int clip;
+		unsigned int clip; /* clip len  should  be unsigned by now */
 
 		/* recalculate clipping if there are tabs in the stream */
 		if (pipe->recalclen) {
@@ -155,15 +160,25 @@ int print_stream(pipe_t *pipe)
 	if (regmatch && !config.dumbterm) {
 		/* Prevent colorizing the next line if regexp matches */
 		if (pipe->buf[ccount-1] == '\n') {
-			int rlen = strlen(pipe->escreset) + 1;
+			int rlen = pipe->escresetlen + 1;
 			strcpy(&pipe->buf[ccount-1],  pipe->escreset);
 			strcpy(&pipe->buf[ccount-1 + rlen],  "\n");
 			esclength+=rlen;
+			regmatch = 1;
+		} else {
+			regmatch = 0;
 		}
 	}
 
 	write(pipe->cfhno, buf, ccount + esclength);
 
+	if (regmatch && !config.dumbterm) {
+		/* Restore \n */
+		if (regmatch) {
+			pipe->buf[ccount-1] = '\n';
+		}
+	}
+
 	pipe->recalclen = 0;
 
 	return 1;
@@ -200,17 +215,17 @@ int print_summary(void)
 
 	fprintf(stderr, " stderr lines;");
 
-       if (WIFEXITED(app.exit_state)) {
-			 fprintf(stderr, " exit state %u\n",
-				WEXITSTATUS(app.exit_state));
+	if (WIFEXITED(app.exit_state)) {
+		fprintf(stderr, " exit state %u\n",
+			WEXITSTATUS(app.exit_state));
 	} else if (WIFSIGNALED(app.exit_state)) {
-			 fprintf(stderr, " signal %d\n",
-				WTERMSIG(app.exit_state));
+		fprintf(stderr, " signal %d\n",
+			WTERMSIG(app.exit_state));
 	} else {
-			 fprintf(stderr, " unknown exit state\n");
+		fprintf(stderr, " unknown exit state\n");
 	}
 
-       /* Show a message if we changed the logfile name */
+	/* Show a message if we changed the logfile name */
 	if ((logfile.appendnr && config.warnlogfilelock)
 	    || config.printlogname)
 		fprintf(stderr, "Logapp logfile is: %s\n", logfile.name);
@@ -323,11 +338,11 @@ int prepare_buffer(pipe_t* pipe)
 {
 	if (pipe->blen) {
 		/* length = buffer length + escape prefix + escape postfix */
-		int len = pipe->blen + strlen(pipe->esccolor)
-			+ strlen(pipe->escreset) + 1;
+		int len = pipe->blen + pipe->esccolorlen + 1;
 
 		if (!(pipe->buf = (char*)malloc(len))) {
-			error("out of memory trying to allocate %d byte "
+			error_outofmemory();
+			error("trying to allocate %d byte "
 			      "%s buffer\n", pipe->blen + 1, pipe->name);
 			return -1;
 		}
@@ -337,7 +352,8 @@ int prepare_buffer(pipe_t* pipe)
 		if (!pipe->charbased) {
 			len = pipe->rblen + 1;
 			if (!(pipe->rdbuf.start = (char*)malloc(len))) {
-				error("out of memory trying to allocate %d "
+				error_outofmemory();
+				error("trying to allocate %d "
 				      "byte %s  read buffer\n",
 				      pipe->rblen + 1, pipe->name);
 				return -1;
@@ -353,10 +369,12 @@ int prepare_buffer(pipe_t* pipe)
 		 * sequence to end the regexp match background color */
 		strcpy(pipe->buf, pipe->esccolor);
 		free(pipe->esccolor);
-		pipe->bgesccolor = pipe->buf + (pipe->bgesccolor
-			- pipe->esccolor); /* fixup bg color pointer */
+		if (pipe->bgesccolor) {
+			pipe->bgesccolor = pipe->buf + (pipe->bgesccolor
+				- pipe->esccolor); /* fixup bg color pointer */
+		}
 		pipe->esccolor = pipe->buf;
-		pipe->buf += strlen(pipe->esccolor);
+		pipe->buf += pipe->esccolorlen;
 	};
 
 	return 0;
@@ -378,6 +396,7 @@ int handle_cbstream(pipe_t* pipe)
 		if (CONFIG_SUPPORT_PTY && config.ptyremovecr
 		    && config.usepty) {
 			int i;
+			int lbo = 0;
 			char* bufa = pipe->buf;
 			char* bufb = pipe->buf;
 
@@ -388,10 +407,24 @@ int handle_cbstream(pipe_t* pipe)
 				if (*bufa == '\n') {
 					if (*(bufa-1) == '\r')
 						bufb--;
+					if ((config.alignlinebreaks)
+					    && (lbo<LINEBREAKBUFSIZE)) {
+						pipe->linebreakpos[lbo++] = 
+							bufb - pipe->buf + 1;
+					}
 				}
 			}
 			pipe->bfill -= (bufa-bufb);
 			*bufb = *bufa;
+			/* linebreakpos buffer is defined size + 1 for the 0 */
+			pipe->linebreakpos[lbo] = 0;
+
+			/* if there is no linebreak detected so far, add
+			 * length of the packet to the array */
+			if (!pipe->linebreakpos[0]) {
+				pipe->linebreakpos[0] = pipe->bfill;
+				pipe->linebreakpos[1] = 0;
+			}
 
 			/* If there is a lonely CR at the end of the buffer,
 			 * ignore it this time until the next read */
