@@ -33,6 +33,7 @@
 
 
 int show_usage = 0;
+int show_version = 0;
 int show_config = 0;
 
 /* general configuration */
@@ -60,6 +61,7 @@ config_t config = {
 	.maxlogfiles		= 10,
 	.logrename		= 1,
 	.alignlog		= 1,
+	.alignlinebreaks	= 1,
 	.jointimeout		= 0,
 	.configfile		= NULL,
 	.configsection		= NULL,
@@ -99,6 +101,7 @@ pipe_t pstdout = {
 	.esccolor	= NULL,
 	.linecount	= 0,
 	.lineprefix	= NULL,
+	.linebreakpos	= {0},
 	.regexp		= NULL,
 	.regbgcol	= 4,
 	.bgesccolor	= NULL,
@@ -129,6 +132,7 @@ pipe_t pstderr = {
 	.esccolor	= NULL,
 	.linecount	= 0,
 	.lineprefix	= "STDERR: ",
+	.linebreakpos	= {0},
 	.regexp		= NULL,
 	.regbgcol	= 4,
 	.bgesccolor	= NULL,
@@ -198,6 +202,17 @@ const stringvalue_t clipvalues[] = {
 	{ NULL,		0 }
 };
 
+const stringvalue_t multprefixes[] = {
+	{ "",		1 },
+	{ "k",		1000 },
+	{ "M",		1000000 },
+	{ "G",		1000000000 },
+	{ "ki",		1024 },
+	{ "Mi",		1048576 },
+	{ "Gi",		1073741824 },
+	{ NULL,		0 }
+};
+
 /* all possible configuration parameters with help message */
 arglist_t arglist[] = {
 	/* General options */
@@ -205,6 +220,8 @@ arglist_t arglist[] = {
 		"General options:", 0 },
 	{ '?',	"help",		NULL,		TNONE,	&show_usage,
 		"show this help", 0 },
+	{ '\0',	"version",	NULL,		TNONE,	&show_version,
+		"show version information", 0 },
 	{ '\0', "configfile",   "FILE",		TSTRING,
 		&config.custconfigfile,	"configuration file", 0 },
 	{ '\0',	"showconfig",	NULL,		TNONE,	&show_config,
@@ -266,6 +283,8 @@ arglist_t arglist[] = {
 		"use one line for every write in charbased mode", 0 },
 	{ '\0', "jointimeout",	"TIME",	TUINT, &config.jointimeout,
 		"join timeout for packets with alignlog active", 0 },
+	{ '\0', "alignlinebreaks",	"BOOL",	TBOOL, &config.alignlinebreaks,
+		"align logged line breaks in charbased mode", 0 },
 	{ 't',  "logtime",	"BOOL",	TBOOL, &config.logtime,
 		"add timestamp to each logged line", 0 },
 	{ '\0',  "logreltime",	"BOOL",	TBOOL, &config.logreltime,
@@ -328,7 +347,7 @@ const int arglistsize = ((int)(sizeof(arglist)/sizeof(arglist_t)));
 
 void show_configuration(void)
 {
-	int i;
+	unsigned int i;
 
 	message("%s %s\n\n", EXECUTABLE, VERSION);
 	message("current configuration:\n");
@@ -342,7 +361,8 @@ void show_configuration(void)
 		message("  config search path (%d)  %s\n", i,
 			configsearchpath[i]);
 	};
-	message("  activate section        %s\n", config.configsection);
+	message("  activate section        %s\n",
+		config.configsection?config.configsection:"<none>");
 	message("  application logfile     %s\n",
 		config.logname?config.logname:"<none>");
 	message("  append to logfile       %i\n", config.appendlog);
@@ -351,6 +371,7 @@ void show_configuration(void)
 	message("  log environment         %i\n", config.logenv);
 	message("  align log writes left   %i\n", config.alignlog);
 	message("  join timeout            %i\n", config.jointimeout);
+	message("  align line breaks       %i\n", config.alignlinebreaks);
 	message("  lock logfiles           %i\n", config.locklogfile);
 	message("  warning on logfile lock %i\n", config.warnlogfilelock);
 	message("  max. alternate logfiles %u\n", config.maxlogfiles - 1);
@@ -362,14 +383,16 @@ void show_configuration(void)
 	message("\n  application executable  %s\n",
 		app.exe?app.exe:"<none>");
 	message("  application argc        %d\n", app.argc);
-	for (i=0; i<app.argc; i++) {
+	for (i=0; i<(unsigned int)app.argc; i++) {
 		message("  application argv[%d]     %s\n", i,
 			app.argv[i]);
 	}
 
 	message("\n");
-	message("  preexec command         %s\n", config.preexeccmd);
-	message("  postexec command        %s\n", config.postexeccmd);
+	message("  preexec command         %s\n",
+		config.preexeccmd?config.preexeccmd:"<none>");
+	message("  postexec command        %s\n",
+		config.postexeccmd?config.postexeccmd:"<none>");
 
 	message("\n");
 	if (CONFIG_USE_THREADS) {
@@ -463,7 +486,7 @@ char* get_longpath(const char* filename)
 	}
 
 	if (longpath == NULL) {
-		error("out of memory\n");
+		error_outofmemory();
 		return NULL;
 	}
 
@@ -513,43 +536,53 @@ static int fixup_pipe(pipe_t *pipe)
 	}
 
 	if (pipe->lineprefix != NULL) {
-		if (strlen(pipe->lineprefix) == 0) {
+		pipe->lineprefixlen = strlen(pipe->lineprefix);
+		if (pipe->lineprefixlen == 0) {
 			pipe->lineprefix = NULL;
 		}
 	}
 
 	/* build console escape string for this pipe */
-	int len = strlen(pipe->escreset) + 1;
+	int len;
+	pipe->escresetlen = strlen(pipe->escreset);
+	len = pipe->escresetlen;
 
 	if (pipe->fgcol >= 0)
 		len += 5;
-	if (pipe->regbgcol)
+	if (pipe->regbgcol >= 0)
 		len += 5;
 	if (pipe->bold)
 		len += 4;
 
-	pipe->esccolor = malloc(len);
+	pipe->esccolor = malloc(len + 1);
+	pipe->esccolorlen = 0;
 	if (!pipe->esccolor) {
-		error("out of memory\n");
+		error_outofmemory();
 		return -1;
 	}
 
-	sprintf(pipe->esccolor, pipe->escreset);
+	memcpy(pipe->esccolor, pipe->escreset, pipe->escresetlen);
+	pipe->esccolorlen += pipe->escresetlen;
 
-	if (pipe->fgcol >= 0)
-		sprintf(pipe->esccolor+strlen(pipe->esccolor), "\033[3%dm",
+	if (pipe->fgcol >= 0) {
+		sprintf(pipe->esccolor + pipe->esccolorlen, "\033[3%dm",
 			pipe->fgcol);
+		pipe->esccolorlen += 5;
+	}
 
 	if (pipe->regbgcol >= 0) {
-		pipe->bgesccolor = pipe->esccolor+strlen(pipe->esccolor) + 3;
-		sprintf(pipe->esccolor+strlen(pipe->esccolor), "\033[4%dm",
+		pipe->bgesccolor = pipe->esccolor + pipe->esccolorlen + 3;
+		sprintf(pipe->esccolor + pipe->esccolorlen, "\033[4%dm",
 			pipe->bgcol); 
+		pipe->esccolorlen += 5;
 	} else {
 		pipe->bgesccolor = NULL;
 	}
 	
-	if (pipe->bold)
-		sprintf(pipe->esccolor+strlen(pipe->esccolor), "\033[1m");
+	if (pipe->bold) {
+		sprintf(pipe->esccolor + pipe->esccolorlen, "\033[1m");
+		pipe->esccolorlen += 4;
+	}
 
 	if (pipe->regexp) {
 		if (strlen(pipe->regexp)) {
@@ -611,6 +644,20 @@ int fixup_config(void)
 	if (fixup_pipe(app.pstderr))
 		return -1;
 
+	/* Prepare indent string */
+	int maxindentlen = 15;
+	if (app.pstdout->lineprefixlen > app.pstderr->lineprefixlen)
+		maxindentlen+=app.pstdout->lineprefixlen;
+	else
+		maxindentlen+=app.pstderr->lineprefixlen;
+
+	app.logfile->indent = malloc(maxindentlen+1);
+	if (!app.logfile->indent) {
+		error_outofmemory();
+		return -1;
+	}
+	memset(app.logfile->indent, ' ', maxindentlen);
+
 	return 0;
 }
 
@@ -703,9 +750,11 @@ int get_argid(char* arg, int isshort)
 	return -1;
 }
 
-int string2numvalue(char* string, int *value, const stringvalue_t *list )
+int string2numvalue(char* string, int *value, const stringvalue_t *list,
+		    const stringvalue_t *prefixes)
 {
 	char*	p;
+	int tmp_value;
 
 	if (list) {
 		while (strcmp(string, list->string)) {
@@ -721,15 +770,43 @@ int string2numvalue(char* string, int *value, const stringvalue_t *list )
 	}
 
 	errno = 0;
-	*value = (int) strtol(string, &p, 0);
-	if (*p || (p==string)) {
-		error("invalid parameter\n");
-		return -1;
-	}
+	tmp_value = (int) strtol(string, &p, 0);
 	if (errno==ERANGE) {
 		error("value out of range\n");
 		return -1;
 	}
+	if (p==string) {
+		error("invalid parameter value\n");
+		return -1;
+	}
+
+	/* check for prefix */
+	if (prefixes && *p) {
+			error("sdf %d\n", *value);
+		while (strcmp(p, prefixes->string)) {
+			prefixes++;
+			if (prefixes->string == NULL) {
+				break;
+			}
+		}
+		if (prefixes->string != NULL) {
+			signed long long tmp;
+			/* be sure we don't get an integer overflow during
+			 * multiplication */
+			tmp = (signed long long) tmp_value * prefixes->value;
+			if ((tmp<(int)0x80000000)||(tmp>(int)~0x80000000)) {
+				error("value including prefix out of range\n");
+				return -1;
+			}
+
+			tmp_value = (int) tmp;
+		} else {
+			error("invalid prefix for numerical value\n");
+			return -1;
+		}
+	}
+
+	*value = tmp_value;
 
 	return 0;
 }
@@ -737,7 +814,8 @@ int string2numvalue(char* string, int *value, const stringvalue_t *list )
 int get_argvalue(int priority, int argid, int* argp, int argc, char* argv[],
 		 char* value)
 {
-	int	tmpint;
+	int tmpint;
+	int ret;
 
 	if (!arglist[argid].var) {
 		return -1;
@@ -768,16 +846,25 @@ int get_argvalue(int priority, int argid, int* argp, int argc, char* argv[],
 		case TINT:
 		case TUINT:	switch (arglist[argid].type) {
 					case TCLIP:
-						string2numvalue(value, &tmpint,
-								clipvalues);
+						ret = string2numvalue(value,
+								&tmpint,
+								clipvalues,
+								multprefixes);
 						break;
 					case TCOLOR:
-						string2numvalue(value, &tmpint,
-								colorvalues);
+						ret = string2numvalue(value,
+								&tmpint,
+								colorvalues,
+								multprefixes);
 						break;
 					default:
-						string2numvalue(value, &tmpint,
-								NULL);
+						ret = string2numvalue(value,
+								&tmpint,
+								NULL,
+								multprefixes);
+				}
+				if (ret) {
+					return -1;
 				}
 				if ((arglist[argid].type==TUINT)&&(tmpint<0)) {
 					error("negative value not allowed for "
@@ -792,12 +879,15 @@ int get_argvalue(int priority, int argid, int* argp, int argc, char* argv[],
 					break;
 				*((char**)arglist[argid].var) = strdup(value);
 				if (*((char**)arglist[argid].var) == NULL) {
-					error("out of memory\n");
+					error_outofmemory();
 					return -1;
 				}
 				break;
 		case TBOOL:	if (string2numvalue(value, &tmpint,
-						    boolvalues)) {
+						    boolvalues, NULL)) {
+					return -1;
+				}
+				if ((tmpint < 0) || (tmpint > 1)) {
 					error("invalid boolean value\n");
 					return -1;
 				}
@@ -833,7 +923,7 @@ int parse_args(int argc, char* argv[])
 	/* the new argc is still unknown but the old value should be save */
 	app.argv = malloc((argc+1)*sizeof(argv[0]));
 	if (!app.argv) {
-		error("out of memory\n");
+		error_outofmemory();
 		return -1;
 	}
 	app.argc = 1;
@@ -955,7 +1045,7 @@ int parse_configline(char* line)
 		free(appconfig);
 		appconfig = strdup(name);
 		if (!appconfig) {
-			error("out of memory\n");
+			error_outofmemory();
 			return -1;
 		}
 		return 0;
@@ -1002,13 +1092,13 @@ int parse_configline(char* line)
 int get_config(void)
 {
 	FILE* conffile = NULL;
-	int i;
 	char *filename;
 	char linebuf[LINEBUFSIZE];
 
 	/* open config file */
 	if (config.custconfigfile == NULL) {
-		for (i=0; i<CONFIGSEARCHPATH_SIZE; i++) {
+		unsigned int i;
+		for (i=0 ; i < CONFIGSEARCHPATH_SIZE ; i++) {
 			filename = get_longpath(configsearchpath[i]);
 
 			if (filename == NULL)
