@@ -57,6 +57,8 @@ typedef struct {
 	int   alignlog;		/* start a new line for every logged item even
 				   in charbased mode */
 	int   jointimeout;	/* join timeout for char based packets */
+	int   alignlinebreaks;  /* Align lines to the left in the logfile if
+				   timestamps are added */
 	char* configfile;	/* current config file */
 	char* configsection;	/* manually selected section in config file */
 	char* custconfigfile;	/* custom config file */
@@ -102,10 +104,16 @@ typedef struct {
 	int	clip;   /* clip console output after x chars */
 	int	eclip;  /* enable clipping: 0 disable, 1 enable, 2 auto */
 	char*   escreset; /* Escape sequence to reset the console */
+	int	escresetlen;
 	char*   esccolor; /* Escape sequence to set color attribute */
+	int	esccolorlen;
 	
 	int	linecount; /* count line passed to stream */
 	char*   lineprefix; /* prefix for each line in logfile */
+	int	lineprefixlen;
+#define LINEBREAKBUFSIZE	8
+	int	linebreakpos[LINEBREAKBUFSIZE + 1]; /* buffer to store already
+						       detected linebreaks */
 
 	char*   regexp; /* regular expression */
 	regex_t preg;	/* preprocessed regular expression */
@@ -132,6 +140,7 @@ typedef struct {
 	char*   split;		/* prefix for logfile split marker lines */
 	char*   foot;		/* prefix for logfile footer lines */
 	int     addnewline;     /* add newline before next log write */ 
+	char*   indent;		/* prefix for indenting logfile lines */
 } logfile_t; 
 
 extern logfile_t logfile;
@@ -162,10 +171,19 @@ typedef struct {
 extern app_t app;
 
 extern int	show_usage;
+extern int	show_version;
 extern int	show_config;
 
 /* possible argument values */
-typedef enum {TNONE, TSTRING, TINT, TUINT, TBOOL, TCOLOR, TCLIP} t_vartype;
+typedef enum {
+	TBOOL,
+	TCLIP,
+	TCOLOR,
+	TINT,
+	TNONE,
+	TSTRING,
+	TUINT,
+} t_vartype;
 
 typedef struct {
 	char		shrt;	/* one char argument */
