@@ -19,13 +19,17 @@
 #ifndef __LOGAPP_H__
 #define __LOGAPP_H__
 
-#define VERSION		"0.9"
+#define VERSION		"0.10"
 #define EXECUTABLE	"logapp"
 #define AUTHOR		"Michael Brunner <mibru@gmx.de>"
 
-#define message(x...)	fprintf(stderr, x)
-#define error(x...)	fprintf(stderr, EXECUTABLE " error: " x)
-#define warning(x...)	fprintf(stderr, EXECUTABLE " warning: " x)
+#define message(x...)	print(0, x)
+#define warning(x...)	print(1, x)
+#define error(x...)	print(2, x)
+#define STRING_WARNING	" warning: "
+#define STRING_ERROR	" error: "
+void print(int type, const char *format,...);
+void error_outofmemory(void);
 
 #define CONFIG_SUPPORT_PTY	1
 #define CONFIG_USE_THREADS	1
