@@ -23,6 +23,7 @@
 #include <string.h>
 #include <errno.h>
 #include <time.h>
+#include <stdarg.h>
 
 #include <sys/socket.h>
 #include <sys/wait.h>
@@ -37,20 +38,75 @@
 #include "logfile.h"
 #include "capture.h"
 
+void error_outofmemory(void)
+{
+	fprintf(stderr, EXECUTABLE " error: out of memory\n");
+}
+
+#define max(a, b) ((a) > (b) ? (a) : (b))
+void print(int type, const char *format,...)
+{
+	va_list argptr;
+	int format_len;
+	int fmt_len;
+	int exec_len;
+	int warn_len;
+	int err_len;
+	char *fmtstring;
+
+	format_len = strlen(format);
+	exec_len = strlen(EXECUTABLE);
+	warn_len = strlen(STRING_WARNING);
+	err_len = strlen(STRING_ERROR);
+	fmt_len = format_len + exec_len + max(warn_len, err_len) + 1;
+
+	fmtstring = malloc(fmt_len);
+	if (!fmtstring) {
+		error_outofmemory();
+		exit(-1);
+	}
+
+	va_start(argptr, format);
+
+	if (type) {
+		memcpy(fmtstring, EXECUTABLE, exec_len);
+		fmt_len = exec_len;
+	} else {
+		fmt_len = 0;
+	}
+
+	switch (type) {
+		case 1: memcpy(fmtstring + exec_len, STRING_WARNING, warn_len);
+			fmt_len+=warn_len;
+			break;
+		case 2: memcpy(fmtstring + exec_len, STRING_ERROR, err_len);
+			fmt_len+=err_len;
+			break;
+		default: break;
+	}
+	memcpy(fmtstring + fmt_len, format, format_len);
+	*(fmtstring + fmt_len + format_len) = '\0';
+
+	vfprintf(stderr, fmtstring, argptr);
+
+	va_end(argptr);
+
+	free(fmtstring);
+}
+
 #define LINELENGTH 80
 #define DESCOFFSET 33
 void usage(void)
 {
 	int i;
+	unsigned int j;
 	int offset;
 	char linebuf[LINELENGTH + 1];
 
 	message("%s %s\n\n", EXECUTABLE, VERSION);
 	message("Usage: %s [OPTIONS] APPLICATION...\n", EXECUTABLE);
 
-	for (i = 0; i< arglistsize; i++) {
-		int j;
-
+	for (i=0; i<arglistsize; i++) {
 		if ((!arglist[i].shrt) && (!arglist[i].lng)
 		    && (arglist[i].desc)) {
 			message("\n%s\n", arglist[i].desc);
@@ -63,31 +119,37 @@ void usage(void)
 		} else
 			sprintf(&linebuf[2], "    ");
 
+		offset = 5;
 		if (arglist[i].lng)
-			sprintf(&linebuf[5], "--%s", arglist[i].lng);
+			offset += sprintf(&linebuf[5], "--%s", arglist[i].lng);
 		else
-			sprintf(&linebuf[5], "    ");
+			offset += sprintf(&linebuf[5], "    ");
 
-		offset = strlen(linebuf);
 		if (arglist[i].parm)
-			sprintf(&linebuf[offset], "=%s ", arglist[i].parm);
-		offset = strlen(linebuf);
+			offset += sprintf(&linebuf[offset], "=%s ",
+					  arglist[i].parm);
+
+		/* Not checking return values of sprintf above, but be sure
+		 * that offset didn't get smaller before continuing */
+		if (offset < 5)
+			exit(-1);
 
 		for (j=offset; j<DESCOFFSET; j++) {
 			linebuf[j] = ' ';
+			offset++;
 		}
 		linebuf[j] = '\0';
 		if (!arglist[i].desc) {
 			message("%s\n", linebuf);
 			continue;
 		}
-		offset = strlen(linebuf);
 
 		int descoffset = 0;
-		for (j=0; j<(strlen(arglist[i].desc) + 2); j++) {
+		unsigned int desclen;
+		for (j=0; j<((desclen = strlen(arglist[i].desc)) + 2); j++) {
 			if (arglist[i].desc[j] == ' ' ||
 			    (arglist[i].desc[j] == '\n') ||
-			    (j > strlen(arglist[i].desc))) {
+			    (j > desclen)) {
 				if ((offset + (j - descoffset)) >= LINELENGTH) {
 					linebuf[offset] = '\n';
 					linebuf[offset+1] = '\0';
@@ -105,8 +167,6 @@ void usage(void)
 			}
 		}
 
-		
-		offset = strlen(linebuf);
 		message("%s\n", linebuf);
 	}
 
@@ -114,6 +174,16 @@ void usage(void)
 		AUTHOR);
 }
 
+void version(void) {
+	message("%s %s\n", EXECUTABLE, VERSION);
+	message("Build date: %s %s\n\n", __DATE__, __TIME__);
+
+	message("Compiletime configuration:\n");
+	message("  PTY support %s\n", CONFIG_SUPPORT_PTY?"enabled":"disabled");
+	message("  Thread usage %s\n\n",
+		CONFIG_USE_THREADS?"enabled":"disabled");
+}
+
 static void sig_handler(int signr)
 {
 	switch (signr) {
@@ -256,9 +326,16 @@ int main(int argc, char *argv[])
 		exit(EXIT_FAILURE);
 	}
 
-	if (show_usage || (!app.exe && !show_config)) {
-		usage();
-		exit(EXIT_SUCCESS);
+	if (!app.exe && !show_config) {
+		if (show_usage) {
+			usage();
+			exit(EXIT_SUCCESS);
+		}
+
+		if (show_version) {
+			version();
+			exit(EXIT_SUCCESS);
+		}
 	}
 
 	/* get default configuration from file */
